import { registerExtension } from '@codingame/monaco-vscode-api/extensions';

const manifest = {"name":"ng-template","displayName":"Angular Language Service","description":"Editor services for Angular templates","version":"19.2.4","publisher":"Angular","icon":"angular.png","license":"MIT","keywords":["Angular","multi-root ready"],"engines":{"vscode":"^1.74.3"},"capabilities":{"untrustedWorkspaces":{"supported":true},"virtualWorkspaces":{"supported":"limited","description":"The Language Server Protocol does not support remote file systems. Functionality is limited to syntax highlighting only."}},"categories":["Programming Languages"],"contributes":{"submenus":[{"id":"angular.submenu","label":"Angular"}],"configuration":{"title":"Angular Language Service","properties":{"angular.log":{"type":"string","enum":["off","terse","normal","verbose"],"default":"off","description":"Enables logging of the Angular server to a file. This log can be used to diagnose Angular Server issues. The log may contain file paths, source code, and other potentially sensitive information from your project."},"angular.view-engine":{"type":"boolean","default":false,"description":"Use legacy View Engine language service. This option is incompatible with projects using Angular v13 and above."},"angular.enable-strict-mode-prompt":{"type":"boolean","default":true,"markdownDescription":"Prompt to enable the [strictTemplates](https://angular.io/guide/angular-compiler-options#stricttemplates) flag in [angularCompilerOptions](https://angular.io/guide/angular-compiler-options). Note that strict mode is only available when using Ivy."},"angular.trace.server":{"type":"string","scope":"window","enum":["off","messages","verbose"],"default":"off","description":"Traces the communication between VS Code and the Angular language server."},"angular.suggest.includeAutomaticOptionalChainCompletions":{"type":"boolean","default":true,"markdownDescription":"Enable showing completions on potentially undefined values that insert an optional chain call. Requires TS 3.7+ and strict null checks to be enabled."},"angular.suggest.includeCompletionsWithSnippetText":{"type":"boolean","default":true,"markdownDescription":"Enable snippet completions from Angular language server. Requires using TypeScript 4.3+ in the workspace."},"angular.forceStrictTemplates":{"type":"boolean","default":false,"markdownDescription":"Enabling this option will force the language service to use [strictTemplates](https://angular.io/guide/angular-compiler-options#stricttemplates) and ignore the user settings in the `tsconfig.json`."},"angular.suppressAngularDiagnosticCodes":{"type":"string","default":"","markdownDescription":"A comma-separated list of error codes in templates whose diagnostics should be ignored."}}},"grammars":[{"path":"./syntaxes/inline-template.json","scopeName":"inline-template.ng","injectTo":["source.ts"],"embeddedLanguages":{"text.html.derivative":"html","source.css":"css","source.js":"javascript"}},{"path":"./syntaxes/inline-styles.json","scopeName":"inline-styles.ng","injectTo":["source.ts"],"embeddedLanguages":{"source.css.scss":"scss"}},{"path":"./syntaxes/template.json","scopeName":"template.ng","injectTo":["text.html.derivative","source.ts"],"embeddedLanguages":{"text.html":"html","source.css":"css","expression.ng":"javascript"}},{"path":"./syntaxes/template-blocks.json","scopeName":"template.blocks.ng","injectTo":["text.html.derivative","source.ts"],"embeddedLanguages":{"text.html":"html","control.block.expression.ng":"javascript","control.block.body.ng":"html"}},{"path":"./syntaxes/let-declaration.json","scopeName":"template.let.ng","injectTo":["text.html.derivative","source.ts"]},{"path":"./syntaxes/host-object-literal.json","scopeName":"host-object-literal.ng","injectTo":["source.ts"],"embeddedLanguages":{"text.html.derivative":"html","expression.ng":"javascript","source.ts":"typescript"}},{"path":"./syntaxes/template-tag.json","scopeName":"template.tag.ng","injectTo":["text.html.derivative","source.ts"]},{"path":"./syntaxes/expression.json","scopeName":"expression.ng"}]},"repository":{"type":"git","url":"https://github.com/angular/vscode-ng-language-service"}};

const { registerFileUrl, whenReady } = registerExtension(manifest, undefined, {"system":true,"readmePath":"README.md","changelogPath":"CHANGELOG.md"});


registerFileUrl('CHANGELOG.md', new URL('../assets/Angular.ng-template-19.2.4.vsix/CHANGELOG.md', import.meta.url).href, {"mimeType":"text/markdown","size":46278});

registerFileUrl('README.md', new URL('../assets/Angular.ng-template-19.2.4.vsix/README.md', import.meta.url).href, {"mimeType":"text/markdown","size":2996});

registerFileUrl('angular.png', new URL('../assets/Angular.ng-template-19.2.4.vsix/angular.png', import.meta.url).href, {"mimeType":"image/png","size":149516});

registerFileUrl('package.json', new URL('../assets/Angular.ng-template-19.2.4.vsix/package.json', import.meta.url).href, {"mimeType":"application/json","size":10688});

registerFileUrl('syntaxes/expression.json', new URL('../assets/Angular.ng-template-19.2.4.vsix/expression.json', import.meta.url).href, {"mimeType":"application/json","size":29264});

registerFileUrl('syntaxes/host-object-literal.json', new URL('../assets/Angular.ng-template-19.2.4.vsix/host-object-literal.json', import.meta.url).href, {"mimeType":"application/json","size":2266});

registerFileUrl('syntaxes/inline-styles.json', new URL('../assets/Angular.ng-template-19.2.4.vsix/inline-styles.json', import.meta.url).href, {"mimeType":"application/json","size":1951});

registerFileUrl('syntaxes/inline-template.json', new URL('../assets/Angular.ng-template-19.2.4.vsix/inline-template.json', import.meta.url).href, {"mimeType":"application/json","size":1520});

registerFileUrl('syntaxes/let-declaration.json', new URL('../assets/Angular.ng-template-19.2.4.vsix/let-declaration.json', import.meta.url).href, {"mimeType":"application/json","size":1163});

registerFileUrl('syntaxes/template-blocks.json', new URL('../assets/Angular.ng-template-19.2.4.vsix/template-blocks.json', import.meta.url).href, {"mimeType":"application/json","size":2965});

registerFileUrl('syntaxes/template-tag.json', new URL('../assets/Angular.ng-template-19.2.4.vsix/template-tag.json', import.meta.url).href, {"mimeType":"application/json","size":4470});

registerFileUrl('syntaxes/template.json', new URL('../assets/Angular.ng-template-19.2.4.vsix/template.json', import.meta.url).href, {"mimeType":"application/json","size":582});

export { whenReady };
