import { registerExtension } from '@codingame/monaco-vscode-api/extensions';

const manifest = {"name":"cobol-language-support","displayName":"COBOL Language Support","description":"Autocomplete, highlighting and diagnostics for COBOL code and copybooks.","author":"Broadcom","license":"EPL-2.0","version":"2.4.0","preview":false,"publisher":"BroadcomMFD","engines":{"vscode":"^1.82.0"},"repository":{"type":"git","url":"https://github.com/eclipse/che-che4z-lsp-for-cobol.git"},"bugs":{"url":"https://github.com/eclipse/che-che4z-lsp-for-cobol/issues"},"keywords":["mainframe","lsp","cobol","z/os","zowe","zos","ibm","datacom","cics","db2"],"categories":["Programming Languages"],"icon":"resources/logo.png","contributes":{"languages":[{"id":"cobol","extensions":[".cbl",".cob",".cobol",".cpy",".copy"],"aliases":["COBOL"],"configuration":"./syntaxes/lang-config.json","firstLine":"^[0-9 ]{6}([*].*|[ Dd] *([Ii][Dd]([Ee][Nn][Tt][Ii][Ff][Ii][Cc][Aa][Tt][Ii][Oo][Nn])? +[Dd][Ii][Vv][Ii][Ss][Ii][Oo][Nn][. ]|[0-9][0-9] +[@#$A-Z][-A-Z0-9]*[. ]|[*](CBL|PROCESS) ).*)$|^([0-9].{5})?[ ]*(CBL|PROCESS) ","filenamePatterns":["**/broadcommfd.cobol-language-support/*/copybooks/**"]},{"id":"hpcobol","aliases":["HP-COBOL"],"configuration":"./syntaxes/lang-config-hp.json"}],"grammars":[{"language":"cobol","scopeName":"source.cobol","path":"./syntaxes/COBOL.tmLanguage.json","embeddedLanguages":{"meta.embedded.block.html":"html"}},{"scopeName":"meta.embedded.block.cics","path":"./syntaxes/CICS.tmLanguage.json","injectTo":["source.cobol"]},{"scopeName":"meta.embedded.block.sql","path":"./syntaxes/SQL.tmLanguage.json","embeddedLanguages":{"meta.embedded.block.sql":"sql"},"injectTo":["source.cobol"]},{"language":"hpcobol","scopeName":"source.hpcobol","path":"./syntaxes/HP.COBOL.tmLanguage.json","embeddedLanguages":{"meta.embedded.block.html":"html"}}],"configuration":{"title":"COBOL Language Support","properties":{"cobol-lsp.smart-tab":{"oneOf":[{"type":"boolean"},{"type":"array","items":{"type":"number"}},{"type":"object","properties":{"default":{"type":"array","items":{"type":"number"}},"anchors":{"type":"object","patternProperties":{"^.*$":{"type":"array","items":{"type":"number"}}}}}}],"description":"Enable/disable smart tab","default":true},"cobol-lsp.cpy-manager.profiles":{"type":"string","description":"Current default profile for copybook downloader\nZowe Explorer version 1.15.0 or higher is required to download copybooks from the mainframe"},"cobol-lsp.cpy-manager.paths-local":{"type":"array","items":{"type":"string"},"description":"Default list of local paths to search for copybooks. Folders are searched in the order they are listed.","uniqueItems":true},"cobol-lsp.cpy-manager.paths-dsn":{"type":"array","items":{"type":"string"},"description":"Default list of datasets to search for copybooks. Datasets are searched in the order they are listed.\nZowe Explorer version 1.15.0 or higher is required to download copybooks from the mainframe","uniqueItems":true},"cobol-lsp.cpy-manager.paths-uss":{"type":"array","items":{"type":"string"},"description":"Default list of USS paths to search for copybooks. USS paths are searched in the order they are listed.\nZowe Explorer version 1.15.0 or higher is required to download copybooks from the mainframe","uniqueItems":true},"cobol-lsp.cpy-manager.copybook-extensions":{"type":"array","$comment":"Items is converted to an array instead of simple string, to disable the vscode rendering the form for extensions setting, instead shows an hyper link to settings.xml for editing. This is done as empty string is not supported by vscode yet","items":{"type":["string"]},"default":[".CPY",".COPY",".cpy",".copy",""],"description":"List of copybook extensions. Empty string specifies no file extension.","uniqueItems":true},"cobol-lsp.cpy-manager.copybook-file-encoding":{"type":"string","markdownDescription":"The file encoding for copybooks stored in mainframe or USS data sets. You can also edit this parameter in .vscode\\settings.json. Other supported encodings can be found at [supported encodings](https://github.com/ashtuchkin/iconv-lite/wiki/Supported-Encodings) ","enum":["utf8","ucs2","utf16le","ascii","base64","UTF7","UTF7-IMAP","UTF-16BE","UTF-16","UCS-4","UTF-32","UTF-32LE","UTF-32BE","ISO-8859-1","ISO-8859-2","ISO-8859-3","ISO-8859-4","ISO-8859-5","ISO-8859-6","ISO-8859-7","ISO-8859-8","ISO-8859-9","ISO-8859-10","ISO-8859-11","ISO-8859-12","ISO-8859-13","ISO-8859-14","ISO-8859-15","ISO-8859-16","cp437","cp720","cp737","cp775","cp808","cp850","cp852","cp855","cp856","cp857","cp858","cp860","cp861","cp862","cp863","cp864","cp865","cp866","cp869","cp922","cp1046","cp1124","cp1125","cp1129","cp1133","cp1161","cp1162","cp1163","cp1047","cp1140","cp1147","cp1148","cp037","cp500","iso646cn","iso646jp"],"enumDescriptions":["utf8","ucs2","utf16le","ascii","base64","UTF7","UTF7-IMAP","UTF-16BE","UTF-16","UCS-4","UTF-32","UTF-32LE","UTF-32BE","ISO-8859-1","ISO-8859-2","ISO-8859-3","ISO-8859-4","ISO-8859-5","ISO-8859-6","ISO-8859-7","ISO-8859-8","ISO-8859-9","ISO-8859-10","ISO-8859-11","ISO-8859-12","ISO-8859-13","ISO-8859-14","ISO-8859-15","ISO-8859-16","IBM-437","IBM-720","IBM-737","IBM-775","IBM-808","IBM-850","IBM-852","IBM-855","IBM-856","IBM-857","IBM-858","IBM-860","IBM-861","IBM-862","IBM-863","IBM-864","IBM-865","IBM-866","IBM-869","IBM-922","IBM-1046","IBM-1124","IBM-1125","IBM-1129","IBM-1133","IBM-1161","IBM-1162","IBM-1163","IBM-1047","IBM-1140","IBM-1147","IBM-1148","IBM-037","IBM-500","iso646cn","iso646jp"]},"cobol-lsp.cpy-manager.endevor-dependencies":{"type":"string","enum":["ZOWE","ENDEVOR_PROCESSOR"],"enumDescriptions":["Use Zowe Explorer to retrieve copybooks from mainframe data sets and USS files specified in the COBOL Language Support extension settings.","Use Explorer for Endevor to retrieve copybooks from locations specified in the Endevor element processor group."],"description":"The method that is used to retrieve remote copybooks for elements opened in Explorer for Endevor.","default":"ZOWE"},"cobol-lsp.subroutine-manager.paths-local":{"type":"array","items":{"type":"string"},"description":"List of relative local paths to search for subroutines","uniqueItems":true},"cobol-lsp.logging.level.root":{"type":"string","default":"ERROR","enum":["ERROR","WARN","INFO","DEBUG","TRACE","ALL"],"enumDescriptions":["ERROR level logs error events which might be fatal to the application","WARN level logs potentially harmful situations","INFO level logs informational messages which describe overall progress of the application","DEBUG level logs informational events of low importance","TRACE level logs informational events of very low importance","ALL level is used to turn on all logging"],"description":"The logging level for COBOL LS backend"},"cobol-lsp.target-sql-backend":{"type":"string","default":"DB2_SERVER","enum":["DB2_SERVER","DATACOM_SERVER","NONE"],"enumDescriptions":["DB2_SERVER  - IBM DB2 SQL Server","DATACOM_SERVER - CA DATACOM Server","NONE"],"description":"The target SQL backend server which is used on the mainframe"},"cobol-lsp.dialects":{"type":"array","items":{"type":"string"},"description":"List of enabled dialects","uniqueItems":true},"cobol-lsp.cics.translator":{"type":"boolean","default":"true","description":"Enable CICS translator"},"cobol-lsp.serverRuntime":{"type":"string","enum":["JAVA","NATIVE"],"default":"JAVA","enumDescriptions":["JAVA  - Run from a Java Archive (jar). Requires java runtime to be installed","NATIVE - Use platform binary"],"description":"COBOL LS server runtime."},"cobol-lsp.formatting":{"type":"string","default":"None","enum":["Uppercase","Lowercase","None"],"description":"Formatting settings"},"cobol-lsp.lspconfig.compiler":{"type":"string","default":"","description":"Name of the COBOL compiler used by the LSPConfig"},"cobol-lsp.lspconfig.preprocessors":{"type":"array","items":{"type":"string"},"description":"List of preprocessors used by the LSPConfig","uniqueItems":true}}},"configurationDefaults":{"[cobol]":{"editor.guides.indentation":false,"editor.rulers":[7,11,72,80],"editor.wordSeparators":"!\"#$%&'()*+,./:;<=>?@[\\]^`{|}~"},"[hpcobol]":{"editor.guides.indentation":false,"editor.wordSeparators":"!\"#$%&'()*+,./:;<=>?@[\\]^`{|}~"}},"jsonValidation":[{"fileMatch":"/.cobolplugin/pgm_conf.json","url":"./schema/pgm_conf.schema.json"},{"fileMatch":"/.cobolplugin/proc_grps.json","url":"./schema/proc_grps.schema.json"}]},"overrides":{"isexe":"^3.1.1"}};

const { registerFileUrl, whenReady } = registerExtension(manifest, undefined, {"system":true,"readmePath":"readme.md","changelogPath":"changelog.md"});


registerFileUrl('changelog.md', new URL('../assets/broadcomMFD.cobol-language-support-2.4.0.vsix/changelog.md', import.meta.url).href, {"mimeType":"text/markdown","size":20262});

registerFileUrl('package.json', new URL('../assets/broadcomMFD.cobol-language-support-2.4.0.vsix/package.json', import.meta.url).href, {"mimeType":"application/json","size":20182});

registerFileUrl('readme.md', new URL('../assets/broadcomMFD.cobol-language-support-2.4.0.vsix/readme.md', import.meta.url).href, {"mimeType":"text/markdown","size":26669});

registerFileUrl('resources/logo.png', new URL('../assets/broadcomMFD.cobol-language-support-2.4.0.vsix/logo.png', import.meta.url).href, {"mimeType":"image/png","size":6250});

registerFileUrl('schema/pgm_conf.schema.json', new URL('../assets/broadcomMFD.cobol-language-support-2.4.0.vsix/pgm_conf.schema.json', import.meta.url).href, {"mimeType":"application/json","size":595});

registerFileUrl('schema/proc_grps.schema.json', new URL('../assets/broadcomMFD.cobol-language-support-2.4.0.vsix/proc_grps.schema.json', import.meta.url).href, {"mimeType":"application/json","size":1471});

registerFileUrl('syntaxes/CICS.tmLanguage.json', new URL('../assets/broadcomMFD.cobol-language-support-2.4.0.vsix/CICS.tmLanguage.json', import.meta.url).href, {"mimeType":"application/json","size":21425});

registerFileUrl('syntaxes/COBOL.tmLanguage.json', new URL('../assets/broadcomMFD.cobol-language-support-2.4.0.vsix/COBOL.tmLanguage.json', import.meta.url).href, {"mimeType":"application/json","size":11146});

registerFileUrl('syntaxes/HP.COBOL.tmLanguage.json', new URL('../assets/broadcomMFD.cobol-language-support-2.4.0.vsix/HP.COBOL.tmLanguage.json', import.meta.url).href, {"mimeType":"application/json","size":10446});

registerFileUrl('syntaxes/SQL.tmLanguage.json', new URL('../assets/broadcomMFD.cobol-language-support-2.4.0.vsix/SQL.tmLanguage.json', import.meta.url).href, {"mimeType":"application/json","size":7633});

registerFileUrl('syntaxes/lang-config-hp.json', new URL('../assets/broadcomMFD.cobol-language-support-2.4.0.vsix/lang-config.json', import.meta.url).href, {"mimeType":"application/json","size":753});

registerFileUrl('syntaxes/lang-config.json', new URL('../assets/broadcomMFD.cobol-language-support-2.4.0.vsix/lang-config.json', import.meta.url).href, {"mimeType":"application/json","size":753});

export { whenReady };
